//---------------------------------------------------------------------
//  Copyright (C) Microsoft Corporation.  All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
//---------------------------------------------------------------------

using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Windows.Forms.Design.Behavior;

using DesignSurfaceExtenderLibrary.Extensions;

namespace ConvexHullDesignSurfaceExtension
{
    /// <summary>
    /// An <see cref="ExtensionGlyph"/> which paints the convex hull
    /// around the controls on the design surface.
    /// </summary>
    /// <remarks>The <see cref="ComputeConvexHull"/> implementation is 
    /// adapted from Carlos Moreno's solution, first published in the
    /// November 1998 and December 1998 issues of C/C++ Users Journal:
    /// http://www.mochima.com/articles/cuj_geometry_article/cuj_geometry_article.html
    /// </remarks>
    public class ConvexHullGlyph : ExtensionGlyph
    {
        private Region convexHullRegion;

        private static Color customColor = Color.FromArgb( 50, Color.Gray );
        private static SolidBrush shadowBrush = new SolidBrush( customColor );

        const int right_turn = -1;


        internal ConvexHullGlyph(
            BehaviorService behaviorService,
            IComponentChangeService changeService,
            ISelectionService selectionService,
            IReferenceService referenceService,
            IServiceProvider serviceProvider,
            Adorner adorner )
            : base(
                null,
                behaviorService,
                changeService,
                selectionService,
                referenceService,
                serviceProvider,
                new ConvexHullBehavior(),
                adorner 
             )
        {
            
        }

        protected override void ComponentChangedEventHandler( object sender, ComponentChangedEventArgs e )
        {
            if( e.Component is Control )
            {
                string propertyName = e.Member.Name;

                if( this.DependentProperties.Contains( propertyName ) )
                {
                    this.ComputeGeometry();

                    if( this.adorner.Enabled )
                    {
                        this.adorner.Invalidate();
                    }
                }
            }
        }

        /// <summary>
        /// Converts the given collection of points to a <see cref="Region"/>.
        /// </summary>
        private static Region ConvertPointsToRegion( List<Point> points )
        {
            Region r = null;

            if( points.Count > 0 )
            {
                GraphicsPath gp = new GraphicsPath();
                gp.AddLines( points.ToArray() );

                r = new Region( gp );
            }

            return r;
        }

        public override void Paint( PaintEventArgs pe )
        {
            if( this.convexHullRegion != null )
            {
                pe.Graphics.FillRegion(
                    ConvexHullGlyph.shadowBrush,
                    this.convexHullRegion );
            }
        }

        /// <summary>
        /// Calculates the convex hull around the controls on the design surface.
        /// </summary>
        protected override void ComputeGeometry()
        {
            List<Point> points = new List<Point>();

            // N.B.: This is not reliable when called from within 
            // an IComponentChangeService event handler. The
            // returned array may not include a newly added control
            // or it may still include a recently deleted control.
            object[] controls = base.referenceService.GetReferences( typeof( Control ) );

            for( int i = 0; i < controls.Length; i++ )
            {
                Control c = controls[i] as Control;

                if( !( c is Form ) )
                {
                    Point p = this.behaviorService.ControlToAdornerWindow( c );

                    Point upperLeft = p;
                    Point upperRight = new Point( p.X + c.Width, p.Y );
                    Point lowerLeft = new Point( p.X, p.Y + c.Height );
                    Point lowerRight = new Point( p.X + c.Width, p.Y + c.Height );

                    points.Add( upperLeft );
                    points.Add( upperRight );
                    points.Add( lowerLeft );
                    points.Add( lowerRight );
                }
            }

            List<Point> convexHullPoints = ComputeConvexHull( points );

            this.convexHullRegion = ConvertPointsToRegion( convexHullPoints );
        }
 
        /// <summary>
        /// Calculates the convex hull around the controls on the design surface.
        /// </summary>
        /// <param name="controlToExclude">A control which is not included in
        /// the convex hull computation.</param>
        /// <remarks>This method is a workaround for the <see cref="IReferenceService"/>
        /// issue. When <see cref="DesignSurfaceExtension.RemoveGlyphsForControl"/> is 
        /// called from within 
        /// <see cref="DesignSurfaceExtension.ComponentRemovedEventHandler"/>, 
        /// the <see cref="IReferenceService.GetReferences"/> still reports the removed glyph.
        /// This overload lets <see cref="DesignSurfaceExtension.RemoveGlyphsForControl"/>  
        /// specify that the recently removed control be excluded from the geometry 
        /// computation.
        /// </remarks>
        internal void ComputeGeometry( Control controlToExclude )
        {
            List<Point> points = new List<Point>();

            object[] controls = base.referenceService.GetReferences( typeof( Control ) );

            ArrayList controlsExcludingGivenControl = new ArrayList( controls );

            if( controlsExcludingGivenControl.Contains( controlToExclude ) )
            {
                controlsExcludingGivenControl.Remove( controlToExclude );

                controls = controlsExcludingGivenControl.ToArray();
            }

            for( int i = 0; i < controls.Length; i++ )
            {
                Control c = controls[i] as Control;

                if( !( c is Form ) )
                {
                    Point p = this.behaviorService.ControlToAdornerWindow( c );

                    Point upperLeft = p;
                    Point upperRight = new Point( p.X + c.Width, p.Y );
                    Point lowerLeft = new Point( p.X, p.Y + c.Height );
                    Point lowerRight = new Point( p.X + c.Width, p.Y + c.Height );

                    points.Add( upperLeft );
                    points.Add( upperRight );
                    points.Add( lowerLeft );
                    points.Add( lowerRight );
                }
            }

            List<Point> convexHullPoints = ComputeConvexHull( points );

            this.convexHullRegion = ConvertPointsToRegion( convexHullPoints );
        }

        public override Cursor GetHitTest( Point p )
        {
            return null;
        }


        protected override ReadOnlyCollection<string> PopulateDependentProperties()
        {
            List<string> properties = new List<string>();

            properties.Add( "Size" );
            properties.Add( "Height" );
            properties.Add( "Width" );
            properties.Add( "Location" );

            return new ReadOnlyCollection<string>( properties );
        }

        ///////////////////////////////////////////////////////////////////////
        #region Convex Hull Implementation

        /// <summary>
        /// Calculates the 2D convex hull of the given collection of points.
        /// </summary>
        /// <param name="points">A list of points for which to find the convex hull. </param>
        /// <returns>A collection of points representing the convex hull of <paramref name="points"/>.</returns>
        /// <remarks>Adapted from
        /// http://www.mochima.com/articles/cuj_geometry_article/cuj_geometry_article.html.
        /// </remarks>
        internal static List<Point> ComputeConvexHull( List<Point> points )
        {
            List<Point> convexHullPoints = new List<Point>();

            if( points.Count > 0 )
            {
                Point p_min_y = points[0];

                for( int i = 1; i < points.Count; i++ )
                {
                    if( points[i].Y < p_min_y.Y )
                    {
                        p_min_y = points[i];
                    }
                }

                convexHullPoints.Add( p_min_y );

                Point p = p_min_y;
                Point new_p = Point.Empty;

                do
                {
                    if( p != points[0] )
                    {
                        new_p = points[0];
                    }
                    else
                    {
                        new_p = points[1];
                    }

                    for( int i = 0; i < points.Count; i++ )
                    {

                        if( turn( p, new_p, points[i] ) == right_turn )
                        {
                            new_p = points[i];
                        }

                    }

                    if( new_p != p_min_y )
                    {
                        convexHullPoints.Add( new_p );
                    }

                    p = new_p;

                } while( new_p != p_min_y );


            }

            return convexHullPoints;
        }

        private static int turn( Point p1, Point p2, Point p3 )
        {

            double signed_area = ( p1.X * ( p2.Y - p3.Y ) +
                                   p2.X * ( p3.Y - p1.Y ) +
                                   p3.X * ( p1.Y - p2.Y ) );

            int orientation = signed_area > 0 ? 1 : ( signed_area < 0 ? -1 : 0 );

            return orientation;
        }

        #endregion Convex Hull Implementation
    }
}
