//---------------------------------------------------------------------
//  Copyright (C) Microsoft Corporation.  All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
//---------------------------------------------------------------------

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Windows.Forms.Design.Behavior;


namespace DesignSurfaceExtenderLibrary.Extensions
{
    /// <summary>
    /// An <see cref="DesignSurfaceExtension"/> for managing 
    /// the <see cref="Control.Anchor"/> property of all controls
    /// on the design surface.
    /// </summary>
    internal class AnchorExtension : DesignSurfaceExtension
    {
        internal AnchorExtension( IServiceProvider serviceProvider )
            : base( serviceProvider )
        {

        }

        /// <summary>
        /// Creates four <see cref="AnchorGlyph"/> instances and 
        /// adds them to the given adorner's <see cref="Adorner.Glyphs"/> collection. 
        /// </summary>
        /// <param name="c">The <see cref="Control"/> which is related to 
        /// the new <see cref="AnchorGlyph"/> instances.</param>
        protected override void AddGlyphsForControl( Control c )
        {
            if( c != null )
            {
                if( !( c is Form ) )
                {
                    Adorner a = this.adornerValue;

                    a.Glyphs.Add( new AnchorGlyph(
                        c,
                        AnchorStyles.Left,
                        base.BehaviorService,
                        base.ComponentChangeService,
                        base.SelectionService,
                        base.ReferenceService,
                        base.ServiceProvider,
                        a ) );

                    a.Glyphs.Add( new AnchorGlyph(
                        c,
                        AnchorStyles.Top,
                        base.BehaviorService,
                        base.ComponentChangeService,
                        base.SelectionService,
                        base.ReferenceService,
                        base.ServiceProvider,
                        a ) );

                    a.Glyphs.Add( new AnchorGlyph(
                        c,
                        AnchorStyles.Right,
                        base.BehaviorService,
                        base.ComponentChangeService,
                        base.SelectionService,
                        base.ReferenceService,
                        base.ServiceProvider,
                        a ) );

                    a.Glyphs.Add( new AnchorGlyph(
                        c,
                        AnchorStyles.Bottom,
                        base.BehaviorService,
                        base.ComponentChangeService,
                        base.SelectionService,
                        base.ReferenceService,
                        base.ServiceProvider,
                        a ) );
                }
            }

        }

        /// <summary>
        /// Removes the <see cref="AnchorGlyph"/> instances associated with
        /// the given control from the <see cref="Adorner.Glyphs"/> collection.
        /// </summary>
        /// <param name="c">The <see cref="Control"/> associated with 
        /// the <see cref="AnchorGlyph"/> instances.</param>
        protected override void RemoveGlyphsForControl( Control c )
        {
            if( c != null )
            {
                for( int i = this.adornerValue.Glyphs.Count - 1; i >= 0; i-- )
                {
                    AnchorGlyph g = this.adornerValue.Glyphs[i] as AnchorGlyph;

                    if( object.ReferenceEquals( c, g.RelatedControl ) )
                    {
                        this.adornerValue.Glyphs.Remove( g );
                    }
                }
            }
        }

        public override DesignerActionList ActionList
        {
            get
            {
                if( this.actionListValue == null )
                {
                    this.actionListValue = new AnchorActionList( this );
                }

                return this.actionListValue;
            }
        }
    }
}
